/* Copyright (c) 1997-1999 Miller Puckette.
 * For information on usage and redistribution, and for a DISCLAIMER OF ALL
 * WARRANTIES, see the file, "LICENSE.txt," in this distribution.  */


#include "m_pd.h"
#include "bigorsmall.h"

/* ---------------- lopass~ - 1-pole lopass filter. ----------------- */

typedef struct lopassctl
    {
        t_sample c_x;
        t_sample c_coef;
    } t_lopassctl;

typedef struct siglopass
    {
        t_object x_obj;
        t_float x_sr;
        t_float x_hz;
        t_lopassctl x_cspace;
        t_lopassctl *x_ctl;
        t_float x_f;
    } t_siglopass;

t_class *siglopass_class;

static void siglopass_ft1(t_siglopass *x, t_floatarg f);

static void *siglopass_new(t_floatarg f)
{
    t_siglopass *x = (t_siglopass *)pd_new(siglopass_class);
    inlet_new(&x->x_obj, &x->x_obj.ob_pd, gensym("float"), gensym("ft1"));
    outlet_new(&x->x_obj, &s_signal);
    x->x_sr = 44100;
    x->x_ctl = &x->x_cspace;
    x->x_cspace.c_x = 0;
    siglopass_ft1(x, f);
    x->x_f = 0;
    return (x);
}

static void siglopass_ft1(t_siglopass *x, t_floatarg f)
{
    if (f < 0) f = 0;
    x->x_hz = f;
    x->x_ctl->c_coef = f * (2 * 3.14159) / x->x_sr;
    if (x->x_ctl->c_coef > 1)
        x->x_ctl->c_coef = 1;
    else if (x->x_ctl->c_coef < 0)
        x->x_ctl->c_coef = 0;
}

static void siglopass_clear(t_siglopass *x, t_floatarg q)
{
    x->x_cspace.c_x = 0;
}

static t_int *siglopass_perform(t_int *w)
{
    t_sample *in = (t_sample *)(w[1]);
    t_sample *out = (t_sample *)(w[2]);
    t_lopassctl *c = (t_lopassctl *)(w[3]);
    int n = (t_int)(w[4]);
    int i;
    t_sample last = c->c_x;
    t_sample coef = c->c_coef;
    t_sample feedback = 1 - coef;
    for (i = 0; i < n; i++)
        last = *out++ = coef * *in++ + feedback * last;
    if (PD_BIGORSMALL(last)) last = 0;  // PD_BIGORSMALL from m_pd.h
    // if (bigorsmall(last)) last = 0;  // bigorsmall from bigorsmall.h
    c->c_x = last;
    return (w+5);
}

static void siglopass_dsp(t_siglopass *x, t_signal **sp)
{
    x->x_sr = sp[0]->s_sr;
    siglopass_ft1(x,  x->x_hz);
    dsp_add(siglopass_perform, 4,
            sp[0]->s_vec, sp[1]->s_vec, 
            x->x_ctl, sp[0]->s_n);
    
}

void lopass_tilde_setup(void)
{
    siglopass_class = class_new(gensym("lopass~"), (t_newmethod)siglopass_new, 0,
                             sizeof(t_siglopass), 0, A_DEFFLOAT, 0);
    CLASS_MAINSIGNALIN(siglopass_class, t_siglopass, x_f);
    class_addmethod(siglopass_class, (t_method)siglopass_dsp, gensym("dsp"), 0);
    class_addmethod(siglopass_class, (t_method)siglopass_ft1,
                    gensym("ft1"), A_FLOAT, 0);
    class_addmethod(siglopass_class, (t_method)siglopass_clear, gensym("clear"), 0);
}